@@ -335,6 +335,14 @@ Application::Application ():
 		OnDeleteSignal = Object::CreateNewSignalId ();
 		OnThemeChangedSignal = Object::CreateNewSignalId ();
 
+		/* get the theme style for labels so that tools buttons colors might
+		be adapted to the current theme */
+		GtkWidget *w=gtk_label_new("");
+		GtkSettings *st = gtk_settings_get_default();
+		m_Style = gtk_rc_get_style(w);
+		gtk_widget_destroy (w);
+
+		// load plugins
 		Plugin::LoadPlugins ();
 		m_bInit = true;
 	}
@@ -543,7 +551,9 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 		}
 	}
 	if (file_type < 0 || (!bSave && (file_type > CHEMISTRY))) {
-		char *mess = g_strdup_printf (_("Sorry, format %s not supported!\nFailed to load %s."), mime_type, filename);
+		char *unescaped = g_uri_unescape_string (filename, NULL);
+		char *mess = g_strdup_printf (_("Sorry, format %s not supported!\nFailed to load %s."), mime_type, unescaped);
+		g_free (unescaped);
 		GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
 													mess);
 		gtk_dialog_run (GTK_DIALOG (message));
@@ -572,7 +582,9 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 		err = g_file_query_exists (file, NULL);
 		gint result = GTK_RESPONSE_YES;
 		if (err) {
-			gchar * message = g_strdup_printf (_("File %s\nexists, overwrite?"), filename2.c_str ());
+			char *unescaped = g_uri_unescape_string (filename2.c_str (), NULL);
+			gchar * message = g_strdup_printf (_("File %s\nexists, overwrite?"), unescaped);
+			g_free (unescaped);
 			GtkDialog* Box = GTK_DIALOG (gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
 			gtk_window_set_icon_name (GTK_WINDOW (Box), "gchempaint");
 			result = gtk_dialog_run (Box);
@@ -585,7 +597,9 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 				GError *error = NULL;
 				g_file_delete (file, NULL, &error);
 				if (error) {
-					gchar * message = g_strdup_printf (_("Error while processing %s:\n%s"), filename2.c_str (), error->message);
+					char *unescaped = g_uri_unescape_string (filename2.c_str (), NULL);
+					gchar * message = g_strdup_printf (_("Error while processing %s:\n%s"), unescaped, error->message);
+					g_free (unescaped);
 					g_error_free (error);
 					GtkDialog* Box = GTK_DIALOG (gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
 					gtk_window_set_icon_name (GTK_WINDOW (Box), "gchempaint");
@@ -804,7 +818,9 @@ void Application::OpenWithBabel (string const &filename, const gchar *mime_type,
 		default:
 			throw (num); //this should not occur
 		}
-		message = gtk_message_dialog_new (NULL, (GtkDialogFlags) 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess, g_uri_unescape_string (filename.c_str (), NULL));
+		char *unescaped = g_uri_unescape_string (filename.c_str (), NULL);
+		message = gtk_message_dialog_new (NULL, (GtkDialogFlags) 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess, unescaped, NULL);
+		g_free (unescaped);
 		gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
 		g_signal_connect_swapped (G_OBJECT (message), "response", G_CALLBACK (gtk_widget_destroy), G_OBJECT (message));
 		gtk_widget_show(message);
@@ -938,7 +954,9 @@ void Application::OpenGcp (string const &filename, Document* pDoc)
 		default:
 			throw (num); //this should not occur
 		}
-		message = gtk_message_dialog_new(NULL, (GtkDialogFlags) 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess, g_uri_unescape_string (filename.c_str (), NULL));
+		char *unescaped = g_uri_unescape_string (filename.c_str (), NULL);
+		message = gtk_message_dialog_new (NULL, (GtkDialogFlags) 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess, unescaped, NULL);
+		g_free (unescaped);
 		gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
 		g_signal_connect_swapped (G_OBJECT (message), "response", G_CALLBACK (gtk_widget_destroy), G_OBJECT (message));
 		gtk_widget_show(message);
@@ -990,8 +1008,22 @@ static void on_tool_changed (GtkAction *action, GtkAction *current, Application*
 
 void Application::OnToolChanged (GtkAction *current)
 {
-	if (m_pActiveTool)
-		m_pActiveTool->Activate(false);
+	char const *name = gtk_action_get_name (current);
+	if (m_pActiveTool) {
+		if (m_pActiveTool->GetName () == name)
+			return;
+		if (!m_pActiveTool->Activate(false)) {
+			GSList *list = gtk_radio_action_get_group (GTK_RADIO_ACTION (current));
+			while (list) {
+				if (m_pActiveTool->GetName () == gtk_action_get_name ((GtkAction *) list->data)) {
+					gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (list->data), true);
+					break;
+				}
+				list = list->next;
+			}
+			return;
+		}
+	}
 	m_pActiveTool = m_Tools[gtk_action_get_name (current)];
 	Tools *ToolsBox = dynamic_cast<Tools*> (GetDialog ("tools"));
 	if (ToolsBox)
@@ -1064,17 +1096,46 @@ void Application::AddActions (GtkRadioActionEntry const *entries, int nb, char c
 		GtkIconSet *set;
 		GtkIconSource *src;
 		while (icons->name) {
+			GdkPixbuf *pixbuf = gdk_pixbuf_new_from_inline (-1, icons->data_24, false, NULL);
 			set = gtk_icon_set_new ();
 			src = gtk_icon_source_new ();
+			gtk_icon_source_set_size_wildcarded (src, true);
+			gtk_icon_source_set_state_wildcarded (src, false);
+			gtk_icon_source_set_direction_wildcarded (src, true);
+
+			for (int c = 0; c < 5; c++) {
+				GdkPixbuf *icon = gdk_pixbuf_copy (pixbuf);
+				// set the pixbuf color to the corresponding style for the style
+				unsigned char red, blue, green;
+				red = m_Style->fg[c].red >> 8;
+				green = m_Style->fg[c].red >> 8;
+				blue = m_Style->fg[c].red >> 8;
+				unsigned char *line, *cur;
+				line = gdk_pixbuf_get_pixels (icon);
+				int i, j, rows, cols, rowstride;
+				cols = gdk_pixbuf_get_width (icon);
+				rows = gdk_pixbuf_get_height (icon);
+				rowstride = gdk_pixbuf_get_rowstride (icon);
+				for (i = 0; i < rows; i++) {
+					cur = line;
+					line += rowstride;
+					for (j = 0; j < cols; j++) {
+						cur[0] = cur[0] ^ red;
+						cur[1] = cur[1] ^ green;
+						cur[2] = cur[2] ^ blue;
+						cur += 4;
+					}
+				}
+				gtk_icon_source_set_pixbuf (src, icon);
+				gtk_icon_source_set_state (src, static_cast <GtkStateType> (c));
+				gtk_icon_set_add_source (set, src);	/* copies the src */
+				g_object_unref (icon);
+			}
 		
-			gtk_icon_source_set_size_wildcarded (src, TRUE);
-			gtk_icon_source_set_pixbuf (src,
-				gdk_pixbuf_new_from_inline (-1, icons->data_24, FALSE, NULL));
-			gtk_icon_set_add_source (set, src);	/* copies the src */
-		
+			gtk_icon_source_free (src);
 			gtk_icon_factory_add (IconFactory, icons->name, set);	/* keeps reference to set */
 			gtk_icon_set_unref (set);
-			gtk_icon_source_free (src);
+			g_object_unref (pixbuf);
 			icons++;
 		}
 	}
