@@ -228,8 +228,8 @@ general:
 							}
 							bond = reinterpret_cast <Bond*> (m_End->GetNextBond (it));
 						}
-					m_coords[6] = *x2 - BondDist * cosa * tanb - dy * sign;
-					m_coords[7] = *y2 + dx * sign + BondDist * sina * tanb;
+					m_coords[6] = *x2 + BondDist * cosa * tanb - dy * sign;
+					m_coords[7] = *y2 + dx * sign - BondDist * sina * tanb;
 					goto done;
 				}
 			} else if (n1 > 1 && n2 > 0) {
@@ -900,9 +900,11 @@ void Bond::Update(GtkWidget* w) const
 	}
 	path = const_cast <Bond *> (this)->BuildPathDef (pData);
 	obj = g_object_get_data(G_OBJECT(group), "path");
-	g_object_set (obj, "bpath", path, NULL);
-	if (m_type == NormalBondType || m_type == UndeterminedBondType)
-		g_object_set (obj, "width_units", pTheme->GetBondWidth (), NULL);
+	if (obj) {
+		g_object_set (obj, "bpath", path, NULL);
+		if (m_type == NormalBondType || m_type == UndeterminedBondType)
+			g_object_set (obj, "width_units", pTheme->GetBondWidth (), NULL);
+	}
 	gnome_canvas_path_def_unref (path);
 }
 	
