@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * programs/gchemtable-curve.cc 
  *
- * Copyright (C) 2005-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2005-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -24,9 +24,11 @@
 
 #include "config.h"
 #include "gchemtable-curve.h"
+#include "gchemtable-data.h"
 #include "gchemtable-data-allocator.h"
 #include <gcu/chemistry.h>
 #include <gcu/element.h>
+#include <gcu/print-setup-dlg.h>
 #include <goffice/data/go-data-simple.h>
 #include <goffice/gtk/go-graph-widget.h>
 #include <goffice/gtk/goffice-gtk.h>
@@ -47,6 +49,7 @@
 #include <goffice/utils/go-marker.h>
 #include <gsf/gsf-input-memory.h>
 #include <gsf/gsf-output-memory.h>
+#include <gsf/gsf-output-gio.h>
 #include <glib/gi18n.h>
 #include <map>
 #include <cstring>
@@ -54,10 +57,6 @@
 using namespace gcu;
 using namespace std;
 
-// FIXME "the following lines should be removed for stable releases"
-#undef PACKAGE
-#define PACKAGE "gchemutils-unstable" 
-
 map<string, GChemTableCurve*> curves;
 
 void on_show_curve (GObject *obj, char const* name)
@@ -150,14 +149,19 @@ static void on_copy (GtkWidget *widget, GChemTableCurve *curve)
 	curve->OnCopy ();
 }
 
+static void on_file_save_as_image(GtkWidget* widget, GChemTableCurve *curve)
+{
+	curve->GetApplication ()->OnSaveAsImage (curve);
+}
+
 static void on_print (GtkWidget *widget, GChemTableCurve *curve)
 {
-	curve->OnPrint (false);
+	curve->Print (false);
 }
 
 static void on_print_preview (GtkWidget *widget, GChemTableCurve *curve)
 {
-	curve->OnPrint (true);
+	curve->Print (true);
 }
 
 static void on_page_setup (GtkWidget *widget, GChemTableCurve *curve)
@@ -190,6 +194,11 @@ static void on_web (GtkWidget *widget, GChemTableCurve *curve)
 	curve->GetApplication ()->OnWeb ();
 }
 
+static void on_live_assistance (GtkWidget *widget, GChemTableCurve *curve)
+{
+	curve->GetApplication ()->OnLiveAssistance ();
+}
+
 static void on_mail (GtkWidget *widget, GChemTableCurve *curve)
 {
 	curve->GetApplication ()->OnMail ();
@@ -207,6 +216,8 @@ static void on_about (GtkWidget *widget, GChemTableCurve *curve)
 
 static GtkActionEntry entries[] = {
   { "FileMenu", NULL, N_("_File") },
+	  { "SaveAsImage", GTK_STOCK_SAVE_AS, N_("Save As _Image..."), "<control>I",
+		  N_("Save the current file as an image"), G_CALLBACK (on_file_save_as_image) },
 	  { "PageSetup", NULL, N_("Page Set_up..."), NULL,
 		  N_("Setup the page settings for your current printer"), G_CALLBACK (on_page_setup) },
 	  { "PrintPreview", GTK_STOCK_PRINT_PREVIEW, N_("Print Pre_view"), NULL,
@@ -229,6 +240,8 @@ static GtkActionEntry entries[] = {
 		  N_("View help for the Curve Window"), G_CALLBACK (on_curve_help) },
 	  { "Web", NULL, N_("Gnome Chemistry Utils on the _web"), NULL,
 		  N_("Browse the Gnome Chemistry Utils's web site"), G_CALLBACK (on_web) },
+	  { "LiveAssistance", NULL, N_("Live assistance"), NULL,
+		  N_("Open the Gnome Chemistry Utils IRC channel"), G_CALLBACK (on_live_assistance) },
 	  { "Mail", NULL, N_("_Ask a question"), NULL,
 		  N_("Ask a question about the Gnome Chemistry Utils"), G_CALLBACK (on_mail) },
 	  { "Bug", NULL, N_("Report _Bugs"), NULL,
@@ -241,6 +254,8 @@ static const char *ui_description =
 "<ui>"
 "  <menubar name='MainMenu'>"
 "    <menu action='FileMenu'>"
+"      <menuitem action='SaveAsImage'/>"
+"		<separator/>"
 "      <menuitem action='PageSetup'/>"
 "      <menuitem action='PrintPreview'/>"
 "      <menuitem action='Print'/>"
@@ -256,43 +271,20 @@ static const char *ui_description =
 "    <menu action='HelpMenu'>"
 "      <menuitem action='Help'/>"
 "      <menuitem action='CurveHelp'/>"
-"      <placeholder name='mail'/>"
-"      <placeholder name='web'/>"
-"      <placeholder name='bug'/>"
+"      <menuitem action='Mail'/>"
+"      <menuitem action='Web'/>"
+"      <menuitem action='LiveAssistance'/>"
+"      <menuitem action='Bug'/>"
 "      <menuitem action='About'/>"
 "    </menu>"
 "  </menubar>"
 "</ui>";
 
-static const char *ui_mail_description =
-"<ui>"
-"  <menubar name='MainMenu'>"
-"    <menu action='HelpMenu'>"
-"      <placeholder name='mail'>"
-"        <menuitem action='Mail'/>"
-"      </placeholder>"
-"    </menu>"
-"  </menubar>"
-"</ui>";
-
-static const char *ui_web_description =
-"<ui>"
-"  <menubar name='MainMenu'>"
-"    <menu action='HelpMenu'>"
-"      <placeholder name='web'>"
-"        <menuitem action='Web'/>"
-"      </placeholder>"
-"      <placeholder name='bug'>"
-"        <menuitem action='Bug'/>"
-"      </placeholder>"
-"    </menu>"
-"  </menubar>"
-"</ui>";
-
 GChemTableCurve::GChemTableCurve (GChemTableApp *App, char const *name):
-	Dialog (App, GLADEDIR"/curve.glade", "curvedlg")
+	Dialog (App, GLADEDIR"/curve.glade", "curvedlg", App),
+	Printable (),
+	m_Guru (NULL)
 {
-	m_Name = name;
 	m_GraphBox = glade_xml_get_widget (xml, "vbox1");
 	GtkUIManager *ui_manager = gtk_ui_manager_new ();
 	GtkActionGroup *action_group = gtk_action_group_new ("MenuActions");
@@ -307,23 +299,21 @@ GChemTableCurve::GChemTableCurve (GChemTableApp *App, char const *name):
 		g_error_free (error);
 		exit (EXIT_FAILURE);
 	}
-	if (m_App->HasWebBrowser () && !gtk_ui_manager_add_ui_from_string (ui_manager, ui_web_description, -1, &error)) {
-		g_message ("building menus failed: %s", error->message);
-		g_error_free (error);
-	}
-	if (m_App->HasMailAgent () && !gtk_ui_manager_add_ui_from_string (ui_manager, ui_mail_description, -1, &error)) {
-		g_message ("building menus failed: %s", error->message);
-		g_error_free (error);
-	}
 	GtkWidget *bar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
 	gtk_box_pack_start (GTK_BOX (m_GraphBox), bar, FALSE, FALSE, 0);
 	m_GraphWidget = go_graph_widget_new (NULL);
 	gtk_widget_set_size_request (m_GraphWidget, 400, 250);
 	gtk_widget_show (m_GraphWidget);
 	gtk_box_pack_end (GTK_BOX (m_GraphBox), m_GraphWidget, TRUE, TRUE, 0);
+	m_Graph = go_graph_widget_get_graph (GO_GRAPH_WIDGET (m_GraphWidget));
 	GogChart *chart = go_graph_widget_get_chart (GO_GRAPH_WIDGET (m_GraphWidget));
+	if (!name) {
+		OnProperties ();
+		return;
+	}
 	GogPlot *plot = (GogPlot *) gog_plot_new_by_name ("GogXYPlot");
 	gog_object_add_by_name (GOG_OBJECT (chart), "Plot", GOG_OBJECT (plot));
+	m_Name = name;
 	// Create a series for the plot and populate it with some simple data
 	GogSeries *series = gog_plot_new_series (plot);
 	double *yvals = g_new0 (double, MAX_ELT);
@@ -502,62 +492,23 @@ GChemTableCurve::GChemTableCurve (GChemTableApp *App, char const *name):
 	gog_dataset_set_dim (GOG_DATASET (obj), GOG_AXIS_ELEM_MAX, data, &error);
 	data = go_data_vector_val_new (yvals, MAX_ELT, g_free);
 	gog_series_set_dim (series, 1, data, &error);
+	gog_series_set_dim (series, 0, gct_data_vector_get_from_name (_("Atomic number")), &error);
 	obj = gog_object_get_child_by_role (GOG_OBJECT (chart),
 			gog_object_find_role_by_name (GOG_OBJECT (chart), "X-Axis"));
 	data = go_data_scalar_str_new ("Z", FALSE);
 	label = (GogObject*) g_object_new (GOG_LABEL_TYPE, NULL);
 	gog_dataset_set_dim (GOG_DATASET (label), 0, data, &error);
 	gog_object_add_by_name (obj, "Label", label);
-	m_Graph = go_graph_widget_get_graph (GO_GRAPH_WIDGET (m_GraphWidget));
-	// Initialize print settings
-	m_PageSetup = gtk_page_setup_new ();
-	m_PrintSettings = gtk_print_settings_new ();
 }
 
 GChemTableCurve::~GChemTableCurve ()
 {
+	if (m_Guru)
+		gtk_widget_destroy (m_Guru);
 	curves.erase (m_Name);
 }
 
-static void begin_print (GtkPrintOperation *print, GtkPrintContext *context, gpointer data)
-{
-	gtk_print_operation_set_n_pages (print, 1);
-}
-
-static void draw_page (GtkPrintOperation *print, GtkPrintContext *context, gint page_nr,gpointer data)
-{
-	((GChemTableCurve *) data)->DoPrint (print, context);
-}
-
-void GChemTableCurve::OnPrint (bool preview)
-{
-	GtkPrintOperation *print;
-	GtkPrintOperationResult res;
-
-	print = gtk_print_operation_new ();
-	gtk_print_operation_set_use_full_page (print, false);
-
-    gtk_print_operation_set_print_settings (print, m_PrintSettings);
-    gtk_print_operation_set_default_page_setup (print, m_PageSetup);
-
-	g_signal_connect (print, "begin_print", G_CALLBACK (begin_print), NULL);
-	g_signal_connect (print, "draw_page", G_CALLBACK (draw_page), this);
-	
-	res = gtk_print_operation_run (print,
-								   (preview)? GTK_PRINT_OPERATION_ACTION_PREVIEW:
-								   			GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
-								   GTK_WINDOW (dialog), NULL);
-
-	if (res == GTK_PRINT_OPERATION_RESULT_APPLY) {
-		if (m_PrintSettings != NULL)
-			g_object_unref (m_PrintSettings);
-		m_PrintSettings = GTK_PRINT_SETTINGS (g_object_ref (gtk_print_operation_get_print_settings (print)));
-	}
-
-	g_object_unref (print);
-}
-
-void GChemTableCurve::DoPrint (GtkPrintOperation *print, GtkPrintContext *context)
+void GChemTableCurve::DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const
 {
 	cairo_t *cr;
 	gdouble width, height;
@@ -565,18 +516,38 @@ void GChemTableCurve::DoPrint (GtkPrintOperation *print, GtkPrintContext *contex
 	cr = gtk_print_context_get_cairo_context (context);
 	width = gtk_print_context_get_width (context);
 	height = gtk_print_context_get_height (context);
-	gog_graph_render_to_cairo (m_Graph, cr, width, height);
+
+	int w, h; // size in points
+	w = m_GraphWidget->allocation.width;
+	h = m_GraphWidget->allocation.height;
+	switch (GetScaleType ()) {
+	case GCU_PRINT_SCALE_NONE:
+		break;
+	case GCU_PRINT_SCALE_FIXED:
+		w *= Printable::GetScale ();
+		h *= Printable::GetScale ();
+		break;
+	case GCU_PRINT_SCALE_AUTO:
+		if (GetHorizFit ())
+			w = width;
+		if (GetVertFit ())
+			h = height;
+		break;
+	}
+	double x = 0., y = 0.;
+	if (GetHorizCentered ())
+		x = (width - w) / 2.;
+	if (GetVertCentered ())
+		y = (height - h) / 2.;
+	cairo_save (cr);
+	cairo_translate (cr, x, y);
+	gog_graph_render_to_cairo (m_Graph, cr, w, h);
+	cairo_restore (cr);
 }
 
 void GChemTableCurve::OnPageSetup ()
 {
-	GtkPageSetup *setup = gtk_print_run_page_setup_dialog (
-											dialog,
-											m_PageSetup,
-											m_PrintSettings
-										);
-	g_object_unref (m_PageSetup);
-	m_PageSetup = setup;
+	new PrintSetupDlg (m_App, this);
 }
 
 void GChemTableCurve::OnCopy ()
@@ -591,6 +562,8 @@ static void
 graph_user_config_free_data (gpointer data,
 					  GClosure *closure)
 {
+	GChemTableCurve *curve = gct_control_gui_get_owner (GCT_CONTROL_GUI (data));
+	curve->ClearGuru ();
 	g_object_unref (data);
 	closure->data = NULL;
 }
@@ -604,15 +577,22 @@ on_update_graph (GogGraph *graph, gpointer data)
 	curve->SetGraph (graph);
 }
 
+static void
+on_guru_help (Application *app)
+{
+	app->OnHelp ("customize-curve");
+}
+
 void GChemTableCurve::OnProperties ()
 {
 	GctControlGUI *tcg = GCT_CONTROL_GUI (g_object_new (GCT_CONTROL_GUI_TYPE, NULL));
 	gct_control_gui_set_owner (tcg, this);
 	GClosure *closure = g_cclosure_new (G_CALLBACK (on_update_graph), tcg,
 					(GClosureNotify) graph_user_config_free_data);
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), 
-								  GTK_WINDOW (gog_guru (m_Graph, GOG_DATA_ALLOCATOR (tcg),
-		      											NULL, closure)));
+	m_Guru = gog_guru (m_Graph, GOG_DATA_ALLOCATOR (tcg), NULL, closure);
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (m_Guru));
+	g_signal_connect_swapped (G_OBJECT (gog_guru_get_help_button (m_Guru)), "clicked", G_CALLBACK (on_guru_help), m_App);
+	gtk_widget_show (m_Guru);
 	g_closure_sink (closure);
 }
 
@@ -625,3 +605,20 @@ void GChemTableCurve::SetGraph (GogGraph *graph)
 	m_Graph = go_graph_widget_get_graph (GO_GRAPH_WIDGET (m_GraphWidget));
 }
 
+void GChemTableCurve::SaveAsImage (string const &filename, char const *mime_type, unsigned width, unsigned height) const
+{
+	char *fname = go_mime_to_image_format (mime_type);
+	GOImageFormat format = go_image_get_format_from_name ((fname)? fname: filename.c_str ());
+	if (format == GO_IMAGE_FORMAT_UNKNOWN)
+		return;
+	GError *error = NULL;
+	GsfOutput *output = gsf_output_gio_new_for_uri (filename.c_str (), &error);
+	if (error) {
+		g_error_free (error);
+		return;
+	}
+	GogGraph *graph = gog_graph_dup (m_Graph);
+	gog_graph_set_size (graph, width, height);
+	gog_graph_export_image (graph, format, output, -1., -1.);
+	g_object_unref (graph);
+}
